package service_award_dividend

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/implement"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"math/rand"
	"time"
	"xorm.io/xorm"
)

func Init(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md2.RedisDataBase)
	return
}

func CollectDividendData(eg *xorm.Engine, masterId string, ch *rabbit.Channel) (err error) {
	honeyDividendBasicDb := implement.NewHoneyDividendBasicDb(eg)
	base, _ := honeyDividendBasicDb.GetHoneyDividendBasic()
	if base.IsOpen != 1 {
		err = errors.New("蜂蜜分红未开启！")
		return
	}

	//1、统计当前`现有蜂蜜数量`
	var sql1 = fmt.Sprintf("SELECT SUM(amount) AS total FROM `user_virtual_amount` WHERE coin_id = %d", base.CoinId)
	results1, _ := db.QueryNativeString(eg, sql1)
	nowHoneys := results1[0]["total"] //当前蜂蜜总量

	//2、统计当前`可以分红`的用户数量
	var sql2 = fmt.Sprintf("SELECT COUNT(*)AS total FROM `user_virtual_amount` uva LEFT JOIN user u on u.uid=uva.uid WHERE uva.coin_id = %d AND uva.amount >= %s and u.level >= %d", base.CoinId, base.EveryMonthExchangeOneCashDestroyHoneyValues, base.DividendVipLevel)
	results2, _ := db.QueryNativeString(eg, sql2)
	nowUsers := results2[0]["total"] //当前用户总量

	//3、更新基础设置配置
	base.NowHoneyValues = nowHoneys
	_, err = honeyDividendBasicDb.UpdateHoneyDividendBasic(base, "now_honey_values")
	if err != nil {
		return
	}

	//4、计算数据
	honeyDividendsRecords := new(models.HoneyDividendsRecords)
	has, err := eg.Where("state =?", 0).Desc("id").Get(honeyDividendsRecords)
	if err != nil {
		return
	}
	if has == false {
		fmt.Println(">>>>>>>>>>>当前没有需要分红的记录")
		return nil
	}
	honeyDividendsRecordsAmountValue, _ := decimal.NewFromString(honeyDividendsRecords.Amount)
	nowHoneyValue, _ := decimal.NewFromString(nowHoneys)
	unitPrice := honeyDividendsRecordsAmountValue.Div(nowHoneyValue)

	//5、进行分红(推入mq处理)
	var page = 1
	var pageSize = 100
	for {
		var list []model.UserVirtualAmount
		err = eg.Where("coin_id = ? and amount >= ?", base.CoinId, base.EveryMonthExchangeOneCashDestroyHoneyValues).Limit(pageSize, (page-1)*pageSize).Asc("id").Find(&list)
		if err != nil {
			fmt.Println("err:::::1111", err)
			return
		}
		if len(list) <= 0 {
			break
		}

		for _, v := range list {
			var m model.User
			has1, err1 := eg.Where("uid = ?", v.Uid).Get(&m)
			if err1 != nil || has1 == false {
				continue
			}
			if m.Level >= base.DividendVipLevel {
				tmpValue, _ := decimal.NewFromString(v.Amount)
				amount := unitPrice.Mul(tmpValue).String()
				//TODO::推入rabbitmq 异步处理
				ch.Publish(md2.HoneyDividendExchange, md2.HoneyDividendForUserDividend{
					RecordsId:          honeyDividendsRecords.Id,
					MasterId:           masterId,
					Uid:                v.Uid,
					Amount:             amount,
					DestroyHoneyValues: base.EveryMonthExchangeOneCashDestroyHoneyValues,
					CoinId:             base.CoinId,
					DividendVipLevel:   base.DividendVipLevel,
				}, md2.HoneyDividendRoutKeyForUserDividend)
			}
		}
		page++
		time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	}

	//6、修改分红记录数据
	honeyDividendsRecords.TotalHoneyValues = nowHoneys
	honeyDividendsRecords.UserNums = zhios_order_relate_utils.StrToInt(nowUsers)
	honeyDividendsRecords.State = 1
	honeyDividendsRecordsDb := implement.NewHoneyDividendsRecordsDb(eg)
	_, err = honeyDividendsRecordsDb.UpdateHoneyDividendsRecords(honeyDividendsRecords, "total_honey_values", "user_nums", "state")
	if err != nil {
		return err
	}

	return
}

func DealDividendReward(session *xorm.Session, data md2.HoneyDividendForUserDividend) error {
	userData, err := db.UserFindByID(session.Engine(), data.Uid)
	if err != nil || userData == nil {
		return errors.New("查询用户数据失败")
	}
	if userData.Level < data.DividendVipLevel {
		//TODO::用户等级不够，不参与分红
		return nil
	}

	//1、进行蜂蜜销毁
	err = rule.DealUserCoin(session, md2.DealUserCoinReq{
		Kind:         "sub",
		Mid:          data.MasterId,
		Title:        md2.HoneyDividendForDestroy,
		TransferType: md2.HoneyDividendForDestroyForUserVirtualCoinFlow,
		OrdId:        "",
		CoinId:       data.CoinId,
		Uid:          data.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(data.DestroyHoneyValues),
	})
	if err != nil {
		return err
	}

	//2、给用户加上余额
	orderType := enum.FinUserFlowOrderActionString(md2.HoneyDividendForRewardForFinUserFlow)
	err = svc.DealUserAmount(session, md2.DealUserAmount{
		Kind:      "add",
		Mid:       data.MasterId,
		Title:     md2.HoneyDividendForReward,
		OrderType: orderType,
		OrdAction: md2.HoneyDividendForRewardForFinUserFlow,
		OrdId:     zhios_order_relate_utils.Int64ToStr(data.RecordsId),
		Uid:       data.Uid,
		Amount:    zhios_order_relate_utils.StrToFloat64(data.Amount),
		Num:       "",
	})
	if err != nil {
		return err
	}

	//3、进行数据记录
	honeyDividendsRecordsWithFlowDb := implement.NewHoneyDividendsRecordsWithFlowDb(nil)
	_, err = honeyDividendsRecordsWithFlowDb.HoneyDividendsRecordsWithFlowInsert(session, &models.HoneyDividendsRecordsWithFlow{
		RecordsId: int(data.RecordsId),
		Uid:       data.Uid,
		Amount:    data.Amount,
		CreateAt:  time.Now().Format("2006-01-02 15:04:05"),
		UpdateAt:  time.Now().Format("2006-01-02 15:04:05"),
	})
	if err != nil {
		return err
	}
	return nil
}
